﻿using System;
using System.Globalization;
using UnityEngine;
using UnityEngine.XR;
using System.Text;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.XR;
using MacGruber;
using customevent;
using MVR.FileManagementSecure;
using Random = System.Random;
using Random2 = UnityEngine.Random;

namespace SPQR
{
    public class Footsteps : MVRScript
    {
		
        string atomid;

        private bool gameIsVR;
		private EventTrigger myEventTrigger;
        private EventTrigger myEventTriggerLeft;
        private EventTrigger myEventTriggerRight;
		
        GameObject playerHands;
        float collisionThreshold = 0.05f;
		float collisionCooldown = 0.5f;
		JSONStorableFloat treshold1;
		JSONStorableFloat treshold2;
		JSONStorableFloat jfloat2;
        JSONStorableStringChooser soundChooser;

		string SavePath = "Saves\\PluginData\\Footsteps";
        string SaveExtension = "json";
		
		private float myClock = -1.0f;
		
		int titles = 0;
        protected JSONStorableString myString;

        Dictionary<string, int> sounds = new Dictionary<string, int>(){
            {"boots", 9},
            {"carpet",6 },
            {"concrete", 11},
            {"dirt",5 },
            {"grass",1 },
            {"grass2", 6},
            {"gravel", 1},
            {"gritty", 10},
            {"hard", 11},
            {"heels1", 13},
            {"heels2", 3},
            {"heels3", 1},
            {"heels4", 10},
            {"ice", 2},
            {"squeaky", 30},
            {"step", 2},
            {"step2", 4},
            {"wood1", 17},
            {"wood2", 4},
            {"wood3", 20}
        };


		Dictionary<string, JSONStorableBool> enableFeatures = new Dictionary<string, JSONStorableBool>();

		JSONStorableString variableThisElement;
		JSONStorableString variableColliderGroup;
		JSONStorableString variableColliderId;
		JSONStorableString variableColliderElement;
		
		
		string[] variableThisElements;
		string[] variableColliderGroups;
		string[] variableColliderIds;
		string[] variableColliderElements;
		
		JSONStorableString collisionsLog;
		JSONStorableBool collisionsLogBool;
		
		JSONStorableString collisionsPreview;
		UIDynamicTextField dtext4;

		Dictionary<string, UIDynamicButton> buttons = new Dictionary<string, UIDynamicButton>();

        private bool initiated = false;

		public void updateVariables(){
			
			if(variableThisElement.val.Trim() == "")
				variableThisElements = new string[0];
			else 
				variableThisElements =  variableThisElement.val.Split('|').Select(k => k.Trim()).ToArray();
			
			if(variableColliderGroup.val.Trim() == "")
				variableColliderGroups = new string[0];
			else 
				variableColliderGroups = variableColliderGroup.val.Split('|').Select(k => k.Trim()).ToArray();
			
			if(variableColliderId.val.Trim() == "")
				variableColliderIds = new string[0];
			else 
				variableColliderIds = variableColliderId.val.Split('|').Select(k => k.Trim()).ToArray();
			
			if(variableColliderElement.val.Trim() == "")
				variableColliderElements = new string[0];
			else 
				variableColliderElements = variableColliderElement.val.Split('|').Select(k => k.Trim()).ToArray();
			
		}

        string getIdFootstepsLeft(){
            var personId = containingAtom.uid;
            return personId+".footsteps.left";
        }
        string getIdFootstepsRight(){
            var personId = containingAtom.uid;
            return personId+".footsteps.right";
        }

        void FootstepsInit(){
            //SuperController.LogMessage("FootstepsInit"+Time.time);
            try{
                var footstepsLeftId = getIdFootstepsLeft();
                var footstepsRightId = getIdFootstepsRight();
                if (SuperController.singleton.GetAtomByUid(footstepsLeftId) == null){
                    // SuperController.singleton.RemoveAtom(SuperController.singleton.GetAtomByUid(footstepsLeftId), true);
                    StartCoroutine(CoroutineCreateAtomLeft(footstepsLeftId));
                }
                if (SuperController.singleton.GetAtomByUid(footstepsRightId) == null){
                    // SuperController.singleton.RemoveAtom(SuperController.singleton.GetAtomByUid(footstepsRightId), true);
                    StartCoroutine(CoroutineCreateAtomRight(footstepsRightId));
                }
            } catch(Exception e){
                //  SuperController.LogError("err2:"+e);
            }
            initiated = true;
            
        }

        private IEnumerator CoroutineCreateAtomLeft(string atomid){
            yield return SuperController.singleton.AddAtomByType("AudioSource", atomid);
            FreeControllerV3 lController = containingAtom.GetStorableByID("lFootControl") as FreeControllerV3;
            FootstepsAlign(atomid,lController, "lFoot");
        }
         private IEnumerator CoroutineCreateAtomRight(string atomid){
            yield return SuperController.singleton.AddAtomByType("AudioSource", atomid);
            FreeControllerV3 rController = containingAtom.GetStorableByID("rFootControl") as FreeControllerV3;
            FootstepsAlign(atomid,rController, "rFoot");
        }

        void FootstepsAlign(string atomid, FreeControllerV3 footController,string rb){
            Atom RAE = SuperController.singleton.GetAtomByUid(atomid);
            RAE.ClearParentAtom();
            RAE.SelectAtomParent(containingAtom);

            FreeControllerV3 raeController = RAE.GetStorableByID("control") as FreeControllerV3;
            
            RAE.mainController.currentPositionState = FreeControllerV3.PositionState.On;
            RAE.mainController.currentRotationState = FreeControllerV3.RotationState.On;
            raeController.transform.position = footController.transform.position;
            raeController.transform.rotation = footController.transform.rotation;
            RAE.mainController.currentPositionState = FreeControllerV3.PositionState.ParentLink;
            RAE.mainController.currentRotationState = FreeControllerV3.RotationState.ParentLink;
            raeController.SetLinkToAtom(containingAtom.uid);
            raeController.SetLinkToRigidbodyObject(rb);
            raeController.currentPositionState = FreeControllerV3.PositionState.ParentLink;
        }

        public override void Init()
        {
            if(containingAtom == null || containingAtom.type != "Person"){
                SuperController.LogError("Footsteps plugin must be added on a Person atom!");
                return;
            }

            SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;

            Utils.OnInitUI(CreateUIElement);
            gameIsVR = XRSettings.isDeviceActive && !string.IsNullOrEmpty(XRSettings.loadedDeviceName);
    
			atomid = containingAtom.uid;
			
			JSONStorableString logo = new JSONStorableString("logostring", "\n<size=40><b>Footsteps</b></size>");
            UIDynamicTextField logotext = CreateTextField(logo);
            logotext.UItext.supportRichText = true;
            logotext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logotext.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
            logotext.height = 60f;
            logotext.UItext.alignment = TextAnchor.UpperLeft;


            JSONStorableString logor = new JSONStorableString("logospace", "\n<size=30><b>v2</b></size>");
            UIDynamicTextField logortext = CreateTextField(logor, true);
            logortext.UItext.supportRichText = true;
            logortext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logortext.backgroundColor = new Color(0f, 0f, 0f, 0f);
            logortext.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
            logortext.height = 60f;
            logortext.UItext.alignment = TextAnchor.UpperRight;

            Utils.SetupSpacer(this, 10, false);
            Utils.SetupSpacer(this, 10, true);

	
			// CreateTitle("THIS ATOM",35,false);

             List<string> choices = new List<string>();
            choices.Add("None");
            foreach(var item in sounds){
                choices.Add(item.Key);
            }
            soundChooser = new JSONStorableStringChooser("Sounds", choices, "None", "Sounds", DummyChooserCallback);
            RegisterStringChooser(soundChooser);
            UIDynamicPopup udp = CreateScrollablePopup(soundChooser, false);
            udp.labelWidth = 300f;

			
			variableThisElement = new JSONStorableString("Element:", "rFoot|lFoot", UpdateVarValue);
            variableThisElement.storeType = JSONStorableParam.StoreType.Full;
			RegisterString(variableThisElement);
			//Utils.SetupTextInput(this, "Element ", variableThisElement, false);
			

	//		CreateTitle("COLLIDED ATOM",35,false);
			
			variableColliderGroup = new JSONStorableString("Collider Group","Slate|Floor|WoodPanel|ReflectiveSlate|ReflectiveWoodPanel", UpdateVarValue);
            variableColliderGroup.storeType = JSONStorableParam.StoreType.Full;
			RegisterString(variableColliderGroup);
		//	Utils.SetupTextInput(this, "Group ", variableColliderGroup, false);
			
			variableColliderId = new JSONStorableString("Collider Atom","", UpdateVarValue);
            variableColliderId.storeType = JSONStorableParam.StoreType.Full;
			RegisterString(variableColliderId);
		//	Utils.SetupTextInput(this, "Atom Name", variableColliderId, false);
			
			variableColliderElement = new JSONStorableString("Collider Element","", UpdateVarValue);
            variableColliderElement.storeType = JSONStorableParam.StoreType.Full;
			RegisterString(variableColliderElement);
		//	Utils.SetupTextInput(this, "Element ", variableColliderElement, false);
			
			
			Utils.SetupSpacer(this, 66, false);
			
			collisionsLogBool = new JSONStorableBool("Enable log", true);
            collisionsLogBool.storeType = JSONStorableParam.StoreType.Full;
			RegisterBool(collisionsLogBool);
			//CreateToggle(collisionsLogBool, false);
				
			// collisionsLog = new JSONStorableString("collisionsLog", "");
            // collisionsLog.storeType = JSONStorableParam.StoreType.Full;
            // UIDynamicTextField dtext3 = CreateTextField(collisionsLog, false);
            // dtext3.UItext.supportRichText = true;
            // dtext3.backgroundColor = new Color(0.42f, 0.03f, 0.51f, 0.7f);
            // dtext3.textColor = new Color(1f, 1f, 1f, 0.9f);
            // dtext3.height = 600f;
			
			treshold1 = new JSONStorableFloat("Impact Min", 0.05f, BlendCallback, 0f, 10f, true);
            treshold1.storeType = JSONStorableParam.StoreType.Full;
			RegisterFloat(treshold1);
			UIDynamicSlider dslider = CreateSlider(treshold1,true);
			
			treshold2 = new JSONStorableFloat("Impact Max", 10f, BlendCallback, 0f, 10f, true);
            treshold2.storeType = JSONStorableParam.StoreType.Full;
			RegisterFloat(treshold2);
			UIDynamicSlider dslider2 = CreateSlider(treshold2,true);
			
			jfloat2 = new JSONStorableFloat("Cooldown", collisionCooldown, BlendCallback, 0f, 10f, true);
            jfloat2.storeType = JSONStorableParam.StoreType.Full;
			RegisterFloat(jfloat2);
			UIDynamicSlider dslider3 = CreateSlider(jfloat2,true);
	
			myEventTrigger = new EventTrigger(this, "TriggerOnUpdate");
            Utils.SetupButton(this, "Trigger On Step", myEventTrigger.OpenPanel, true);
         
            myEventTriggerLeft = new EventTrigger(this, "TriggerOnUpdateLeft");
            Utils.SetupButton(this, "Trigger On Left", myEventTriggerLeft.OpenPanel, true);

            myEventTriggerRight = new EventTrigger(this, "TriggerOnUpdateRight");
            Utils.SetupButton(this, "Trigger On Right", myEventTriggerRight.OpenPanel, true);
			
          
			
			Utils.SetupSpacer(this, 66, true);
			collisionsPreview = new JSONStorableString("collisionsPreview", "");
		    dtext4 = CreateTextField(collisionsPreview, true);
            dtext4.UItext.supportRichText = true;
            dtext4.backgroundColor = new Color(1f, 0.33f, 0.33f, 0.7f);
            dtext4.textColor = new Color(1f, 1f, 1f, 0.9f);
            dtext4.height = 100f;

			// buttons.Add("load_preset", CreateButton("Load Preset",true));
            //     buttons["load_preset"].button.onClick.AddListener(() =>
            //     {
            //         FileManagerSecure.CreateDirectory(SavePath);
            //         var shortcuts = FileManagerSecure.GetShortCutsForDirectory(SavePath);
            //         SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, SaveExtension, SavePath, false, true, false, null, false, shortcuts);
            //     });

            //     buttons.Add("save_preset", CreateButton("Save Preset", true));
            //     buttons["save_preset"].button.onClick.AddListener(() =>
            //     {
            //         FileManagerSecure.CreateDirectory(SavePath);
            //         var fileBrowserUI = SuperController.singleton.fileBrowserUI;
            //         fileBrowserUI.SetTitle("Save ticklish preset");
            //         fileBrowserUI.fileRemovePrefix = null;
            //         fileBrowserUI.hideExtension = true;
            //         fileBrowserUI.keepOpen = false;
            //         fileBrowserUI.canCancel = true;
            //         fileBrowserUI.fileFormat = SaveExtension;
            //         fileBrowserUI.defaultPath = SavePath;
            //         fileBrowserUI.showDirs = true;
            //         fileBrowserUI.showFiles = true;
            //         fileBrowserUI.shortCuts = null;
            //         fileBrowserUI.browseVarFilesAsDirectories = false;
            //         fileBrowserUI.SetTextEntry(true);
            //         fileBrowserUI.Show(HandleSavePreset);
            //         fileBrowserUI.fileEntryField.text = (int)(DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalSeconds + "." + SaveExtension;
            //         fileBrowserUI.ActivateFileNameField();
            //     });


            
           
            UIUpdate();
            updateVariables();
            SimpleTriggerHandler.LoadAssets();
        }

        private void UIUpdate()
        {
        }
	

        void Start()
        {	
            
            InvokeRepeating("UpdateSecond", 1f, 1f);
			RefreshColliders();
        }

        void Update()
        {
			myEventTrigger.Update();
			myEventTriggerLeft.Update();
			myEventTriggerRight.Update();
			if (myClock < 0.0f)
				return;

			myClock -= Time.deltaTime;
        }

        void FixedUpdate()
        {
           
        }

        void UpdateSecond(){
            if(IsDisabled()) return;
            if(!initiated){
                FootstepsInit();
            }
        }

        public bool IsDisabled()
        {
            if (SuperController.singleton.freezeAnimationToggle.isOn) return true;
            if (!enabledJSON.val) return true;
            return false;
        }
		
		void OnEnable(){
			RefreshColliders();
			updateVariables();
		}

        void OnDisable()
        {
            containingAtom.GetComponentsInChildren<Rigidbody>().ToList().ForEach(f =>
			{
				try{
				 f.gameObject.GetComponent<TriggerCollide>().OnCollide -= ObserveTrigger;
				} catch(Exception e){
					//SuperController.LogError("err1:"+e);
				}
			});
        }
		void OnReload()
        {
            OnDisable();
        }
	    private void OnAtomRename(string oldid, string newid)
        {
          
            myEventTrigger.SyncAtomNames();
            myEventTriggerLeft.SyncAtomNames();
            myEventTriggerRight.SyncAtomNames();
			if(variableColliderId.val == oldid){
				variableColliderId.val = newid;
			}
            if(atomid != oldid) return;
            atomid = newid;
            var footstepsLeftId = oldid+".footsteps.left";
            var footstepsRightId = oldid+".footsteps.right";

            var footstepsLeftIdnew = newid+".footsteps.left";
            var footstepsRightIdnew = newid+".footsteps.right";

            var leftatom = SuperController.singleton.GetAtomByUid(footstepsLeftId);
            var rightatom = SuperController.singleton.GetAtomByUid(footstepsRightId);
            if (leftatom!= null){
                 SuperController.singleton.RenameAtom(leftatom, footstepsLeftIdnew);
            }
            if (rightatom != null){
                SuperController.singleton.RenameAtom(rightatom, footstepsRightIdnew);
            }
        }
		
		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
            
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
				jc[myEventTriggerLeft.Name] = myEventTriggerLeft.GetJSON(base.subScenePrefix);
				jc[myEventTriggerRight.Name] = myEventTriggerRight.GetJSON(base.subScenePrefix);
			}
			return jc;
		}
		
		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
            

			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				myEventTriggerLeft.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				myEventTriggerRight.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
            
		}

        void OnDestroy()
        {
            SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
            CancelInvoke();  
            Utils.OnDestroyUI();
            OnDisable();
			myEventTrigger.Remove();
			myEventTriggerLeft.Remove();
			myEventTriggerRight.Remove();
        }

	
		public void UpdateVarValue(string s){
			updateVariables();
		}	
		void BlendCallback(JSONStorableFloat jf){
		}
		
	    private void TestTrigger()
        {
           Trigger();

        }
        private void Trigger()
        {
            myEventTrigger.Trigger();
        }

        private void TriggerLeft()
        {
            myEventTriggerLeft.Trigger();
        }
        private void TriggerRight()
        {
            myEventTriggerRight.Trigger();
        }
		
		
        public void RefreshColliders()
        {
			
			containingAtom.GetComponentsInChildren<Rigidbody>().ToList().ForEach(f =>
			{
				try{
				 f.gameObject.GetComponent<TriggerCollide>().OnCollide += ObserveTrigger;
				} catch (Exception e){
					f.gameObject.AddComponent<TriggerCollide>().OnCollide += ObserveTrigger;
				}
			});
	
        }



        void ObserveTrigger(object sender, TriggerEventArgs e){
            Atom parent = null;
			if(myClock > 0f) return;
            foreach (var component in e.collider.gameObject.GetComponentsInParent<Component>())
            {

                var atom = component as Atom;
                if (atom != null)
                {
                    parent = atom;
                    break;
                }
            }
          



            var found = "";
            var components = e.collider.transform.parent.GetComponents<Component>();

            var RB = e.collider.attachedRigidbody;

            foreach (var xatom in SuperController.singleton.GetAtoms())
            {
                var RB2 = xatom.rigidbodies.First();
                if (RB == RB2)
                {
                    found = xatom.name;
                    break;
                }

            }
			


            if (e.evtType == customevent.EventType.HIT)
            {
                float level = e.collision.relativeVelocity.magnitude / 3.0f;
                if (level > treshold1.val && level < treshold2.val)
                {
                    //if (level > 3) level = 3;

                    //SuperController.LogMessage("collide at high level: " + level + ", threshold: " + collisionThreshold.val);
                    if (e.insideTrigger)
                    { }
                    else
                    {
						/*
						foreach(var contactpoint in e.collision.contacts){
							SuperController.LogMessage(contactpoint.thisCollider.attachedRigidbody.name + " with " + contactpoint.otherCollider.attachedRigidbody.name);
						}
						*/
						
                        var xname = found.Length > 0 ? found : parent.name;
                       var xcollider = e.collider.name;
					   if(e.collision.contacts.Count() > 0){
						xcollider = e.collision.contacts[0].otherCollider.attachedRigidbody.name;
					   }
                        RaeClickTouch(sender.ToString(), xname, parent.type, xcollider, level);

                        //SuperController.LogMessage("!!!!!Receiving event " + parent.name + " of " + found + " of " + e.collider.name);
                        // SuperController.LogMessage("Collide with other object");

                    }
                }
                else
                { 
					
				}
            }
            else{}
			
        }


        //////////////////////////////////////////////

        public void RaePreloadSound(string f)
        {

            string path = Utils.GetPackagePath(this) + "Custom/Sounds/Aeternum/" + f;
            string localPath = SuperController.singleton.NormalizeLoadPath(path);
            // SuperController.LogMessage("audio:"+localPath);
            NamedAudioClip nac;
            nac = URLAudioClipManager.singleton.GetClip(localPath);
            if (nac == null)
            {
                URLAudioClip clip = URLAudioClipManager.singleton.QueueClip(SuperController.singleton.NormalizeMediaPath(path));
                nac = URLAudioClipManager.singleton.GetClip(clip.uid);
                //nac.sourceClip.LoadAudioData();

            }


        }

         public void RaeRemovedSound(string f)
        {

            string path = Utils.GetPackagePath(this) + "Custom/Sounds/Aeternum/" + f;
            string localPath = SuperController.singleton.NormalizeLoadPath(path);
            // SuperController.LogMessage("audio:"+localPath);
            NamedAudioClip nac;
            nac = URLAudioClipManager.singleton.GetClip(localPath);
            if (nac != null)
            {
                URLAudioClipManager.singleton.RemoveClip(nac);
            }


        }

		void RaeClickTouch(string sender, string name, string type, string collider, float level){
            if(IsDisabled()) return;
			var slevel =  String.Format("{0:0.000}",level);
			sender = sender.Replace(" (customevent.TriggerCollide)","");
			if(name==containingAtom.name) return;
			// if(collisionsLogBool.val){
			// 	var s = ""+sender + " collision ("+slevel+") with:  \n" + type +":"+name+":"
			// 	+collider + "\n\n" + collisionsLog.val;
				
			// 	if(s.Length > 500){
			// 		s = s.Substring(0,500);
			// 	}
			// 	collisionsLog.val = s;
				
			// //	SuperController.LogMessage("Detected touch from "+sender+" with "+name+" - ("+type+")"+collider + "with level " +level);
				
			// }
			
			if(variableThisElements.Count() > 0){
				if(Array.IndexOf(variableThisElements, sender) == -1) return;
			}
			
			
			if(variableColliderGroups.Count() > 0){
				if(Array.IndexOf(variableColliderGroups, type) == -1) return;
			}
			
			if(variableColliderIds.Count() > 0){
				if(Array.IndexOf(variableColliderIds, name) == -1) return;
			}
			
			
			if(variableColliderElements.Count() > 0){
				if(Array.IndexOf(variableColliderElements, collider) == -1) return;
			}
			
			
			
			

            if(sender=="rToe" || sender=="rBigToe" || sender=="rFoot"){
                collisionsPreview.val = "Right Foot";
                FootstepsRight();
            } else {
                collisionsPreview.val = "Left Foot";
                FootstepsLeft();
            }
            
			dtext4.backgroundColor = new Color(0.33f, 1f, 0.33f, 0.7f);
			StartCoroutine(CoroutineFlashTrigger());
			myClock = jfloat2.val;

			
		}

        string getRandomSound(){
            var count = sounds[soundChooser.val];
            int rand = Random2.Range(1,  count+1);
            var f =  "footsteps/"+soundChooser.val+"/"+soundChooser.val+" ("+rand+").wav";
            //SuperController.LogMessage(f);
            return f;

        }

        void FootstepsLeft(){
            Trigger();
            TriggerLeft();
            if(soundChooser.val == "None") return;

            RaeSound(getIdFootstepsLeft(), getRandomSound());
        }

         void FootstepsRight(){
            Trigger();
            TriggerRight();
            if(soundChooser.val == "None") return;
            RaeSound(getIdFootstepsRight(), getRandomSound());
        }

        public AudioSourceControl RaeAudioSource(string id){
            Atom raeAudio = SuperController.singleton.GetAtomByUid(id);
            AudioSourceControl raeAudioSource = null;
            if (raeAudio != null){
                raeAudioSource = raeAudio.GetStorableByID("AudioSource") as AudioSourceControl;
            }
            return raeAudioSource;
        }

        public void RaeSound(string audiosourceId, string f)
        {   
            if(IsDisabled()) return;
            if(SuperController.singleton.GetAtomByUid(audiosourceId) == null) return;

            // SuperController.LogMessage("RaeSound: " + f);
            AudioSourceControl raeAudioSource = RaeAudioSource(audiosourceId);
            if (raeAudioSource != null)
            {
                //   SuperController.LogMessage("RaeSound ok");
                string path = Utils.GetPackagePath(this) + "Custom/Sounds/Aeternum/" + f;
                string localPath = SuperController.singleton.NormalizeLoadPath(path);
                //  SuperController.LogMessage("audio:"+localPath);
                NamedAudioClip nac;
                nac = URLAudioClipManager.singleton.GetClip(localPath);
                if (nac == null)
                {
                    URLAudioClip clip = URLAudioClipManager.singleton.QueueClip(SuperController.singleton.NormalizeMediaPath(path));
                    nac = URLAudioClipManager.singleton.GetClip(clip.uid);
                }
                if (nac != null)
                {
                    raeAudioSource.audioSource.Stop();
                    // raeAudioSource.audioSource.PlayOneShot(nac.sourceClip);
                    raeAudioSource.audioSource.clip = nac.sourceClip;
                    raeAudioSource.audioSource.Play();

                }
            }
      

        }


		 private IEnumerator CoroutineFlashTrigger(){
		    yield return StartCoroutine(Wait(0.33f));
		    collisionsPreview.val = "";
			dtext4.backgroundColor = new Color(1f, 0.33f, 0.33f, 0.7f);
		 }
		 
		 
		IEnumerator Wait(float seconds)
		{
		yield return new WaitForSeconds(seconds);
		}

        void UpdateMyString(string name)
        {
        }

		private void CreateTitle(string text, int fontsize, bool right,int spacing){
            CreateTitle(text, fontsize, right);
            Utils.SetupSpacer(this, spacing, right);
        }
        private void CreateTitle(string text, int fontsize, bool right){
                JSONStorableString title1 = new JSONStorableString("title"+(++titles), "\n<size="+fontsize+"><b>"+text+"</b></size>");
                UIDynamicTextField title1text = CreateTextField(title1, right);
                title1text.UItext.supportRichText = true;
                title1text.backgroundColor = new Color(0f, 0f, 0f, 0f);
                title1text.textColor = new Color(0.42f, 0.03f, 0.51f, 1f);
                title1text.height = 100f;
                title1text.UItext.alignment = right ? TextAnchor.MiddleRight : TextAnchor.MiddleLeft;
        }

		
        private void HandleLoadPreset(string path)
        {
           if (path == null) return;
            var jc = (JSONClass)LoadJSON(path);
            base.RestoreFromJSON(jc);

			myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix,false, true);
        }

        private void HandleSavePreset(string path)
        {
            SuperController.singleton.fileBrowserUI.fileFormat = null;
            if (path == null) return;
            var jc = GetJSON(true,true,true);
            SaveJSON(jc, path);
        }

        protected void DummyChooserCallback(string s) {
            removeClips();
            if(s == "None") return;
            var count = sounds[s];
            for(var i =1 ; i<=count; i++){
                    RaePreloadSound("footsteps/"+s+"/"+s+" ("+i+").wav");
            }
		}

        void removeClips(){
            foreach(var item in sounds){
                for(var i =1 ; i<=item.Value; i++){
                    RaeRemovedSound("footsteps/"+item.Key+"/"+item.Key+" ("+i+").wav");
                }
            }

        }


     
        


    }



}
